@@ -11,7 +11,7 @@ use Perinci::Object;
 use Perinci::ToUtil;
 use Scalar::Util qw(reftype blessed);
 
-our $VERSION = '0.78'; # VERSION
+our $VERSION = '0.79'; # VERSION
 
 with 'Perinci::To::Text::AddDocLinesRole';
 with 'SHARYANTO::Role::Doc::Section';
@@ -802,11 +802,10 @@ my ($ph1, $ph2); # patch handles
 sub _setup_progress_output {
     my $self = shift;
 
-    require Progress::Any;
     if ($ENV{PROGRESS} // (-t STDOUT)) {
-        require Progress::Any::Output::TermProgressBar;
-        state $out = Progress::Any::Output::TermProgressBar->new;
-        Progress::Any->set_output(output => $out);
+        require Progress::Any::Output;
+        Progress::Any::Output->set("TermProgressBarColor");
+        my $out = $Progress::Any::outputs{''}[0];
         if ($self->{_log_any_app_loaded}) {
             # we need to patch the logger adapters so it won't interfere with
             # progress meter's output
@@ -815,7 +814,17 @@ sub _setup_progress_output {
                 'Log::Log4perl::Appender::Screen', 'log',
                 'replace', sub {
                     my ($self, %params) = @_;
-                    $out->_message($params{message});
+
+                    # clean currently displayed progress bar first
+                    if ($out->{lastlen}) {
+                        print
+                            "\b" x $out->{lastlen},
+                            " " x $out->{lastlen},
+                            "\b" x $out->{lastlen};
+                        undef $out->{lastlen};
+                    }
+
+                    say $params{message};
                 },
             );
             $ph2 = Monkey::Patch::Action::patch_package(
@@ -829,14 +838,23 @@ sub _setup_progress_output {
                         $msg = Term::ANSIColor::colored($msg, $color);
                     }
                     # END copy-paste'ish
-                    $out->_message($msg);
+
+                    # clean currently displayed progress bar first
+                    if ($out->{lastlen}) {
+                        print
+                            "\b" x $out->{lastlen},
+                            " " x $out->{lastlen},
+                            "\b" x $out->{lastlen};
+                        undef $out->{lastlen};
+                    }
+
+                    # XXX duplicated code above, perhaps move this to
+                    # TermProgressBarColor's clean_bar() or something
+
+                    say $msg;
                 },
             );
         }
-    } else {
-        require Progress::Any::Output::Null;
-            Progress::Any->set_output(
-                output => Progress::Any::Output::Null->new);
     }
 }
 
@@ -1223,13 +1241,15 @@ sub run {
 __END__
 =pod
 
+=encoding utf-8
+
 =head1 NAME
 
 Perinci::CmdLine - Rinci/Riap-based command-line application framework
 
 =head1 VERSION
 
-version 0.78
+version 0.79
 
 =head1 SYNOPSIS
 
@@ -1876,5 +1896,8 @@ the same terms as the Perl 5 programming language system itself.
 
 =head1 FUNCTIONS
 
+
+None are exported by default, but they are exportable.
+
 =cut
 
